
#include "Protocol.h"
#include "Serial.h"
#include "Report.h"
#include "System.h"
#include "MotionControl.h"
#include <Arduino.h>

volatile bool grbl_running = false;

void grbl_init() {
  Serial.begin(BAUD_RATE);
  while (!Serial);
  report_init_message();

  motion_init();  // Initialize stepper pinModes
  protocol_init();
}

void run_once() {
  protocol_loop();
}

void protocol_init() {
  grbl_running = true;
  Serial.println("Grbl 1.1h ['$' for help]");
  report_status();
}

void protocol_loop() {
  if (Serial.available()) {
    String line = Serial.readStringUntil('\n');
    line.trim();
    line.toUpperCase();  // Normalize case for command parsing

    if (line.length() == 0) {
      Serial.println("ok");  // ← asegura que UGS no se queje
      return;
    }
    
    if (line == "?") {
      report_status();
    } else if (line == "$I") {
      Serial.println("[VER:1.1h:Barebones]");
      Serial.println("[OPT:VN]");
      Serial.println("ok");
    } else if (line == "$G") {
      Serial.println("[G90 G21 G17 G94 G54]");
      Serial.println("ok");
    } else if (line == "$$") {
      Serial.println("$0=10");
      Serial.println("$1=25");
      Serial.println("$100=240.0");
      Serial.println("$101=340.0");
      Serial.println("$102=100.0");
      Serial.println("$110=1000.0");
      Serial.println("$111=1000.0");
      Serial.println("$112=500.0");
      Serial.println("ok");
    } else if (line == "$X" || line == "$H") {
      Serial.println("ok");
    } else if (line == "G28") {
      motion_go_home();
      Serial.println("ok");
    } else {
      float x = NAN, y = NAN, z = NAN, f = -1;
      bool has_g90 = false, has_g91 = false, has_motion = false;

      char buffer[line.length() + 1];
      strncpy(buffer, line.c_str(), sizeof(buffer));
      buffer[sizeof(buffer) - 1] = '\0';
      char* ptr = buffer;

      while (*ptr) {
        if (*ptr == 'G') {
          ptr++;
          long gcode = strtol(ptr, &ptr, 10);
          switch (gcode) {
            case 0:
            case 1: has_motion = true; break;
            case 90: has_g90 = true; break;
            case 91: has_g91 = true; break;
          }
        } else if (*ptr == 'X') {
          ptr++;
          x = strtof(ptr, &ptr);
        } else if (*ptr == 'Y') {
          ptr++;
          y = strtof(ptr, &ptr);
        } else if (*ptr == 'Z') {
          ptr++;
          z = strtof(ptr, &ptr);
        } else if (*ptr == 'F') {
          ptr++;
          f = strtof(ptr, &ptr);
        } else {
          ptr++;
        }
      }

      if (has_g90) {
        motion_set_absolute_mode(true);
        Serial.println("ok");
      } else if (has_g91) {
        motion_set_absolute_mode(false);
        Serial.println("ok");
      } else if (has_motion) {
        motion_execute_move(x, y, z, f);
        Serial.println("ok");
      } else {
        Serial.println("error: Unsupported command");
      }
    }
  }
}

//may work with UGS send file:
// #include "Protocol.h"
// #include "Serial.h"
// #include "Report.h"
// #include "System.h"
// #include "MotionControl.h"
// #include <Arduino.h>

// volatile bool grbl_running = false;

// void grbl_init() {
//   Serial.begin(BAUD_RATE);
//   while (!Serial);
//   report_init_message();
//   motion_init();
//   protocol_init();
// }

// void run_once() {
//   protocol_loop();
// }

// void protocol_init() {
//   grbl_running = true;
//   Serial.println("Grbl 1.1h ['$' for help]");
//   report_status();
// }
// void process_gcode_line(String line);

// void protocol_loop() {
//   static String input_line = "";

//   while (Serial.available()) {
//     char c = Serial.read();
//     if (c == '\n' || c == '\r') {
//       if (input_line.length() > 0) {
//         input_line.trim();
//         input_line.toUpperCase();
//         process_gcode_line(input_line);
//         input_line = "";
//       }
//     } else {
//       input_line += c;
//     }
//   }
// }

// void process_gcode_line(String line) {
//   if (line == "?") {
//     report_status();
//   } else if (line == "$I") {
//     Serial.println("[VER:1.1h:Barebones]");
//     Serial.println("[OPT:VN]");
//     Serial.println("ok");
//   } else if (line == "$G") {
//     Serial.println("[G90 G21 G17 G94 G54]");
//     Serial.println("ok");
//   } else if (line == "$$") {
//     Serial.println("$0=10");
//     Serial.println("$1=25");
//     Serial.println("$100=240.0");
//     Serial.println("$101=340.0");
//     Serial.println("$102=100.0");
//     Serial.println("$110=1000.0");
//     Serial.println("$111=1000.0");
//     Serial.println("$112=500.0");
//     Serial.println("ok");
//   } else if (line == "$X" || line == "$H") {
//     Serial.println("ok");
//   } else if (line == "G28") {
//     motion_go_home();
//     Serial.println("ok");
//   } else {
//     float x = NAN, y = NAN, z = NAN, f = -1;
//     bool has_g90 = false, has_g91 = false, has_motion = false;

//     char buffer[line.length() + 1];
//     strncpy(buffer, line.c_str(), sizeof(buffer));
//     buffer[sizeof(buffer) - 1] = '\0';
//     char* ptr = buffer;

//     while (*ptr) {
//       if (*ptr == 'G') {
//         ptr++;
//         long gcode = strtol(ptr, &ptr, 10);
//         switch (gcode) {
//           case 0:
//           case 1: has_motion = true; break;
//           case 90: has_g90 = true; break;
//           case 91: has_g91 = true; break;
//         }
//       } else if (*ptr == 'X') {
//         ptr++;
//         x = strtof(ptr, &ptr);
//       } else if (*ptr == 'Y') {
//         ptr++;
//         y = strtof(ptr, &ptr);
//       } else if (*ptr == 'Z') {
//         ptr++;
//         z = strtof(ptr, &ptr);
//       } else if (*ptr == 'F') {
//         ptr++;
//         f = strtof(ptr, &ptr);
//       } else {
//         ptr++;
//       }
//     }

//     if (has_g90) {
//       motion_set_absolute_mode(true);
//       Serial.println("ok");
//     } else if (has_g91) {
//       motion_set_absolute_mode(false);
//       Serial.println("ok");
//     } else if (has_motion) {
//       motion_execute_move(x, y, z, f);
//       Serial.println("ok");
//     } else {
//       Serial.println("error: Unsupported command");
//     }
//   }
// }
