
#include <Arduino.h>
#include "Grbl/System.h"
#include "Grbl/Protocol.h"
#include <ESP32Servo.h>
#include "Grbl/MotionControl.h"

// === Pines de botones ===
#define BTN_X_POS D3
#define BTN_X_NEG D2
#define BTN_Y_POS D1
#define BTN_Y_NEG D9
#define BTN_SET_ORIGIN D4

extern Servo z_servo;

void setup() {
  Serial.begin(115200);
  while (!Serial);

  // Inicializa botones con resistencias pull-up internas
  pinMode(BTN_X_POS, INPUT_PULLUP);
  pinMode(BTN_X_NEG, INPUT_PULLUP);
  pinMode(BTN_Y_POS, INPUT_PULLUP);
  pinMode(BTN_Y_NEG, INPUT_PULLUP);
  pinMode(BTN_SET_ORIGIN, INPUT_PULLUP);

  ESP32PWM::allocateTimer(0);
  ESP32PWM::allocateTimer(1);
  z_servo.attach(Z_SERVO_PIN, 500, 2400); // Rango extendido
  z_servo.write(0);
  delay(1000);

  grbl_init();  // Inicializa GRBL
  Serial.println("Grbl Barebones ready.");
}

// === Control manual con botones ===
void manual_control_loop() {
  static unsigned long last_move = 0;
  const unsigned long debounce_time = 200;  // ms

  if (millis() - last_move < debounce_time) return;

  if (digitalRead(BTN_X_POS) == LOW) {
    Serial.println("🔼 X+ Pressed");
    digitalWrite(X_DIR_PIN, HIGH);
    for (int i = 0; i < 10; i++) {
      digitalWrite(X_STEP_PIN, HIGH);
      delayMicroseconds(1000);
      digitalWrite(X_STEP_PIN, LOW);
      delayMicroseconds(1000);
    }
    last_move = millis();
  }

  if (digitalRead(BTN_X_NEG) == LOW) {
    Serial.println("🔽 X- Pressed");
    digitalWrite(X_DIR_PIN, LOW);
    for (int i = 0; i < 10; i++) {
      digitalWrite(X_STEP_PIN, HIGH);
      delayMicroseconds(1000);
      digitalWrite(X_STEP_PIN, LOW);
      delayMicroseconds(1000);
    }
    last_move = millis();
  }

  if (digitalRead(BTN_Y_POS) == LOW) {
    Serial.println("➡️  Y+ Pressed");
    digitalWrite(Y_DIR_PIN, HIGH);
    for (int i = 0; i < 10; i++) {
      digitalWrite(Y_STEP_PIN, HIGH);
      delayMicroseconds(1000);
      digitalWrite(Y_STEP_PIN, LOW);
      delayMicroseconds(1000);
    }
    last_move = millis();
  }

  if (digitalRead(BTN_Y_NEG) == LOW) {
    Serial.println("⬅️  Y- Pressed");
    digitalWrite(Y_DIR_PIN, LOW);
    for (int i = 0; i < 10; i++) {
      digitalWrite(Y_STEP_PIN, HIGH);
      delayMicroseconds(1000);
      digitalWrite(Y_STEP_PIN, LOW);
      delayMicroseconds(1000);
    }
    last_move = millis();
  }

  if (digitalRead(BTN_SET_ORIGIN) == LOW) {
    Serial.println("🔄 Origin set (manual)");
    current_position[X_AXIS] = 0.0;
    current_position[Y_AXIS] = 0.0;
    last_move = millis();
  }
}


void loop() {
  manual_control_loop();  // ← Control por botones
  run_once();             // ← Ciclo GRBL para comandos G-code
}





// #include <Arduino.h>

// // === Pines para motores ===
// #define X_STEP_PIN D5
// #define X_DIR_PIN  D6
// #define Y_STEP_PIN D7
// #define Y_DIR_PIN  D8

// // === Pines para botones ===
// #define BTN_X_POS D1
// #define BTN_X_NEG D2
// #define BTN_Y_POS D3
// #define BTN_Y_NEG D4
// #define BTN_SET_ORIGIN D9

// void setup() {
//   Serial.begin(115200);
//   while (!Serial);
//   Serial.println("[TEST] Inicio de prueba de botones y motores");

//   // Pines de motor
//   pinMode(X_STEP_PIN, OUTPUT);
//   pinMode(X_DIR_PIN, OUTPUT);
//   pinMode(Y_STEP_PIN, OUTPUT);
//   pinMode(Y_DIR_PIN, OUTPUT);

//   // Pines de botones con resistencia pull-up
//   pinMode(BTN_X_POS, INPUT_PULLUP);
//   pinMode(BTN_X_NEG, INPUT_PULLUP);
//   pinMode(BTN_Y_POS, INPUT_PULLUP);
//   pinMode(BTN_Y_NEG, INPUT_PULLUP);
//   pinMode(BTN_SET_ORIGIN, INPUT_PULLUP);
// }

// void loop() {
//   if (digitalRead(BTN_X_POS) == LOW) {
//     digitalWrite(X_DIR_PIN, HIGH);
//     for (int i = 0; i < 10; i++) {
//       digitalWrite(X_STEP_PIN, HIGH);
//       delayMicroseconds(1000);
//       digitalWrite(X_STEP_PIN, LOW);
//       delayMicroseconds(1000);
//     }
//     Serial.println("🔼 Movimiento directo en X+");
//     delay(200);
//   }

//   if (digitalRead(BTN_X_NEG) == LOW) {
//     digitalWrite(X_DIR_PIN, LOW);
//     for (int i = 0; i < 10; i++) {
//       digitalWrite(X_STEP_PIN, HIGH);
//       delayMicroseconds(1000);
//       digitalWrite(X_STEP_PIN, LOW);
//       delayMicroseconds(1000);
//     }
//     Serial.println("🔽 Movimiento directo en X-");
//     delay(200);
//   }

//   if (digitalRead(BTN_Y_POS) == LOW) {
//     digitalWrite(Y_DIR_PIN, HIGH);
//     for (int i = 0; i < 10; i++) {
//       digitalWrite(Y_STEP_PIN, HIGH);
//       delayMicroseconds(1000);
//       digitalWrite(Y_STEP_PIN, LOW);
//       delayMicroseconds(1000);
//     }
//     Serial.println("➡️  Movimiento directo en Y+");
//     delay(200);
//   }

//   if (digitalRead(BTN_Y_NEG) == LOW) {
//     digitalWrite(Y_DIR_PIN, LOW);
//     for (int i = 0; i < 10; i++) {
//       digitalWrite(Y_STEP_PIN, HIGH);
//       delayMicroseconds(1000);
//       digitalWrite(Y_STEP_PIN, LOW);
//       delayMicroseconds(1000);
//     }
//     Serial.println("⬅️  Movimiento directo en Y-");
//     delay(200);
//   }

//   if (digitalRead(BTN_SET_ORIGIN) == LOW) {
//     Serial.println("🔄 BOTÓN DE ORIGEN PRESIONADO");
//     delay(200);
//   }
// }
